%% 2.7. Una ecuacin difcil de resolver numricamente.
% 
% En este problema se compara el comportamiento de disintos mtodos
% iterativos en la resolucin de la ecuacin x^10 = 1.
% Se destaca la importancia de disponer de estimaciones iniciales
% adecuadas, obtenidas por un mtodo lento, como biseccin, para asegurar
% el buen funcionamiento de mtodos en teora ms rpidos.
%
%
% Tema 2, Problema 7
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% En primer lugar, representamos grficamente la funcin x^10 - 1.
% Observamos que tiene dos races reales en x = -1 y x = 1.

x = linspace(-1.1, 1.1);
f = inline('x.^10-1');
plot([-1.1 1.1], [0 0], x, f(x))
title('y = x^1^0 - 1')
grid


%%
%
% a) Mtodo de biseccin
% Observamos que la funcin cambia de signo en el intervalo [0, 1.5]. A
% partir de este intervalo, el mtodo de biseccin obtiene la respuesta con
% precisin de 10^-6 en 21 iteraciones. El comportamiento de ste mtodo no
% presenta ninguna particularidad.

[sol, x1, incr, iter] = biseccion(f, 0, 1.5, 1e-6, 200)


%%
%
% La grfica muestra la evolucin de los iterados. 

plot(x1, '.-'), grid
title('Biseccin para x^1^0 -1 = 0')
xlabel('Iteracin')
ylabel('Valor del iterado')


%%
%
% b) Mtodo de regula-falsi
% Partiendo del mismo intervalo, son necesarias 158 iteraciones para
% obtener la solucin. Destacamos que en este caso el mtodo de
% regula-falsi converge con mucha ms lentitud que el de biseccin.

[sol, x2, incr, iter] = regulafalsi(f, 0, 1.5, 1e-6, 200)
plot(x2, 'g.-'), grid
title('Regula-falsi para x^1^0 -1 = 0')
xlabel('Iteracin')
ylabel('Valor del iterado')


%%
%
% Para esta ecuacin, regula-falsi converge linealmente con una tasa muy
% prxima a 1, lo que implica gran lentitud de convergencia.

e = abs(diff(x2));
r = e(2:end)./e(1:end-1);
r(end)                 
plot(r,'g.-'), grid
title('Regula-falsi: tasa de convergencia lineal')


%%
%
% c) Mtodo de Newton
% En este caso, el cdigo de la funcin incluir la derivada:

type pnew
plot([-1.1 1.1], [0 0], x, pnew(x))
title('y = x^1^0 - 1')
grid


%%
%
% Partimos de la estimacin inicial x1 = 0.5, pues en x = 0 se anula la
% derivada. 
% Se obtiene sol = 1 en 44 iteraciones. La primera iteracin proporciona
% una estimacin muy lejana que cuesta mucho recuperar.

[sol, x3, incr, k] = newton('pnew', 0.5, 1e-6, 500)
plot(x3, 'r.-'), grid
title('Mtodo de Newton para x^1^0 -1 = 0')
xlabel('Iteracin')
ylabel('Valor del iterado')


%%
%
% Los cocientes de los incrementos de los iterados de Newton se mantienen
% estables en torno a 0.9, lo que corresponde a una convergencia lineal
% bastante lenta. Al acercarse a la solucin, la tasa lineal tiende a 0, lo
% que apunta a una convergencia cuadrtica.

e = abs(diff(x3));
r = e(2:end)./e(1:end-1);
plot(r,'r.-'), grid
title('Mtodo de Newton: tasa de convergencia lineal')


%%
%
% Al acercarse a la solucin, la tasa lineal tiende a 0, lo
% que apunta a una convergencia cuadrtica.

r2 = e(2:end)./e(1:end-1).^2;
plot(r2(end-4:end),'r.-'), grid
title('Mtodo de Newton: tasa de convergencia cuadrtica')


%%
%
% d) Mtodo de la secante
% Este mtodo proporciona una solucin errnea, debido a que el quinto iterado es muy elevado, por lo que en las
% iteraciones 6 y 7, se obtienen estimaciones que estn tan prximas que el
% proceso finaliza.

[sol, x4, incr, k] = secante(f, 0, 1.5, 1e-6, 500)
plot(x4, 'y.-'), grid
title('Mtodo de la secante para x^1^0 -1 = 0')
xlabel('Iteracin')
ylabel('Valor del iterado')

%%
%
% El grfico muestra conjuntamente los iterados de cada mtodo. Se aprecia la diferente velocidad de
% convergencia de cada una de ellas.

plot(x1, '.-')
title('Raz dcima de 1 partiendo de una estimacin poco precisa')
hold on, grid
plot(x2, 'g.-')
plot(x3, 'r.-')
xlabel('Iteraciones')
legend('Biseccin', 'Regula-falsi', 'Newton')
hold off


%%
%
% Como conclusin principal de este problema, destacamos que, para comenzar
% a iterar, se debe tomar una estimacin inicial adecuada. 
% sta puede obtenerse con el mtodo de biseccin, que garantiza un
% intervalo de la longitud deseada conteniendo la solucin.

plot(x1(1:3), '.-'), grid
title('Tres iteraciones de biseccin para x^1^0 -1 = 0')
xlabel('Iteracin')
ylabel('Valor del iterado')


%%
%
% b) Mtodo de regula-falsi
% Partiendo del intervalo obtenido con biseccin, regula-falsi converge
% mucho ms rpido que antes.

[sol, x2, incr, iter] = regulafalsi(f, x1(3), x1(2), 1e-6, 20)
plot(x2, 'g.-'), grid
title('Regula-falsi para x^1^0 -1 = 0')
xlabel('Iteracin')
ylabel('Valor del iterado')


%%
%
% La convergencia es lineal y ms rpida que biseccin, pues la tasa es
% menor que 0.5. 

e = abs(diff(x2));
r = e(2:end)./e(1:end-1)
r2 = e(2:end)./e(1:end-1).^2
plot(r,'g.-'), grid
title('Regula-falsi: tasa de convergencia lineal')


%%
%
% c) Mtodo de Newton
% Ahora converge en pocas iteraciones.

[sol, x3, incr, k] = newton('pnew', x1(3), 1e-6, 10)
plot(x3, 'r.-'), grid
title('Mtodo de Newton para x^1^0 -1 = 0')
xlabel('Iteracin')
ylabel('Valor del iterado')


%%
%
% La convergencia de Newton es cuadrtica, como caba esperar.

e = abs(diff(x3));
r2 = e(2:end)./e(1:end-1).^2
plot(r2,'r.-'), grid
title('Mtodo de Newton: tasa de convergencia cuadrtica')


%%
%
% d) Mtodo de la secante
% Con la estimacin ms precisa, el mtodo converge.

[sol, x4, incr, k] = secante(f, x1(2), x1(3), 1e-6, 20)
plot(x4, 'y.-'), grid
title('Mtodo de la secante para x^1^0 -1 = 0')
xlabel('Iteracin')
ylabel('Valor del iterado')

%%
%
% La convergencia es superlineal, pues la tasa lineal tiende a 0.

e = abs(diff(x4));
r = e(2:end)./e(1:end-1)
r2 = e(2:end)./e(1:end-1).^2
plot(r,'y.-'), grid
title('Secante: tasa de convergencia lineal')


%%
%
% El grfico muestra conjuntamente los iterados de cada mtodo. Se aprecia
% la diferente velocidad de convergencia de cada mtodo, pero todos logan
% el objetivo, al haber partido de una estimacin inicial suficientemente
% precisa.

plot(x1(3:end), '.-')
title('Raz dcima de 1, partiendo de una estimacin adecuada')
hold on, grid
plot(x2, 'g.-')
plot(x3, 'r.-')
plot(x4(2:end), 'y.-')
xlabel('Iteraciones')
legend('Biseccin', 'Regula-falsi', 'Newton', 'Secante')
hold off


%%
%
% FIN del Problema 2.7