%% 5.2. Modelo de poblacin de Malthus-Verhulst. Mtodo de Heun.
% 
% Este problema muestra paso a paso el funcionamiento del mtodo
% de Heun para la resolucin numrica de un problema de valor inicial
% sobre modelos de poblacin. 
% 
%
% Tema 5, Problema 2
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson

%% 
%
% La ecuacin diferencial del modelo de poblacin de Verhulst es 
%           p' = k*p - m*p^2
% con k = 0.05 y m = 0.001. La poblacin inicial es p0 = 100.
% Resolvemos el problema de valor inicial en el intervalo [0, 10] por el
% mtodo de Heun con paso h = 1. 

p0 = 100;
h = 1;
k = 0.05;
m = 0.001;

image(imread('Verhulst.jpeg'))
title('Pierre Verhulst')
axis off

%%
%
% Para estimar p1 por el mtodo de Heun hay que integrar la ecuacin
% diferencial entre 0 y 1 por el mtodo de los trapecios, segn la frmula
%
%       p1 = p0 +h/2*(p'(0,p0)+p'(1,p1))
%
% Como no conocemos p1, hacemos una prediccin de ese valor por el mtodo
% de Euler:

dp0 = k*p0 - m*p0^2;
pp1 = p0 + h*dp0

plot(0, p0, '*')
hold on
plot([0 1], [p0 pp1], 'r', 1, pp1, 'r*')
axis([-0.2 1.2 94 101])
legend('Valor inicial', 'Pendiente en (0, p0)', 'Prediccin por Euler')
title('Modelo de Verhulst: p'' = k*p - m*p^2. Mtodo de Heun')
xlabel('Tiempo'), ylabel('Poblacin')


%%
%
% Con el valor predicho, pp1, calculamos la derivada para evaluar el
% mtodo de los trapecios y obtener el valor corregido de p1.

dp1 = k*pp1 - m*pp1^2;
p1 = p0 + h/2*(dp0+dp1)


%%
%
% Geomtricamente, es como si hacemos otra estimacin con la pendiente en 
% (1, pp1) ...


plot([0 1], [p0 p0 + h*dp1], 'g', 1, p0 + h*dp1, 'g*')
legend('Valor inicial', 'Pendiente en (0, p0)', 'Prediccin por Euler', ...
    'Pendiente en (1, pp1)', 'Prediccin con dp1')


%%
%
% y promediamos ambas estimaciones.

plot(1, p1, '*', [0 1], [p0 p1], 'b')
t1 = text([0 1] + 0.05, [p0 p1] , ['(t_0, p_0)'; '(t_1, p_1)'])
legend('Valor inicial', 'Pendiente en (0, p0)', 'Prediccin por Euler', ...
    'Pendiente en (1, pp1)', 'Prediccin con dp1', ...
    'Pendiente promedio', 'Valor final')


%%
%
% En el segundo paso se toma (1, p1) como valor inicial y se repite el
% proceso. 
% Primero se estima p2 con la pendiente en (1, p1).

dp1 = k*p1 - m*p1^2;
pp2 = p1 + h*dp1;

plot(2, pp2, 'r*', [1 2], [p1 pp2], 'r')
axis([-0.2 2.2 88 101])
title('Mtodo de Heun. Paso 2.')
legend off


%%
%
% Luego, con la pendiente en (2, pp2).

dp2 = k*pp2 - m*pp2^2;

plot(2, p1 + h*dp2, 'g*', [1 2], [p1 p1 + h*dp2], 'g')

%%
%
% Y se promedian por la frmula de los trapecios.

p2 = p1 + h/2*(dp1+dp2)

plot(2, p2, '*', [1 2], [p1 p2], 'b')
t2 = text(2 + 0.05, p2 , '(t_2, p_2)');



%%
%
% El tercer paso se hace del mismo modo.

dp2 = k*p2 - m*p2^2;
pp3 = p2 + h*dp2;
dp3 = k*pp3 - m*pp3^2;
p3 = p2 + h/2*(dp2+dp3)

plot(3, pp3, 'r*', [2 3], [p2 pp3], 'r')
plot(3, p2 + h*dp3, 'g*', [2 3], [p2 p2 + h*dp3], 'g')
plot(3, p3, '*', [2 3], [p2 p3], 'b')
title('Mtodo de Heun. Paso 3')
set(gca, 'XLimMode', 'auto', 'YLimMode', 'auto')
delete(t1, t2)

%%
%
% Y as sucesivamente, hasta llegar a t = 10. Abreviamos la descripcin
% usando la funcin heun.m.

[t,p] = heun('verhulst',0:10,p0);

plot(t,p,'.-')
title('Modelo de Verhulst: p'' = kp - mp^2. Mtodo de Heun.')
xlabel('Tiempo'), ylabel('Poblacin')
hold off
grid


%%
%
% El modelo de Verhulst puede resolverse analticamente. Codificamos la
% solucin del problema de valor inicial considerado en el fichero
% logistic.m.

type logistic


%%
%
% Hallamos los valores de la solucin exacta en los instantes en que
% hemos calculado la solucin aproximada.

solex = logistic(t,p0)';

plot(t, p, '.-', t, solex, '.-')
legend('Sol. numrica (Heun)', 'Sol. analtica')
title('Modelo de Verhulst: p'' = kp - mp^2.')
xlabel('Tiempo'), ylabel('Poblacin')
grid


%%
%
% Obtenemos el error en cada punto y el mximo en el intervalo 

E1 = max(abs(solex-p))

plot(t, abs(p-solex), '.-')
title('Modelo de Verhulst: p'' = kp - mp^2. Error local del mtodo de Heun.')
xlabel('Tiempo'), ylabel('Poblacin')
grid
hold on


%%
%
% Resolvemos ahora el problema con el doble de subintervalos para obtener
% el orden del mtodo. El error de la aproximacin con paso 0.5 es
% practicamente la cuarta parte del obtenido con paso 1, por lo que
% concluimos que el mtodo de Heun es de orden 2.

[t, p] = heun('verhulst', 0:0.5:10, p0);

solex = logistic(t,p0)';
E2 = max(abs(solex-p))

plot(t, abs(p-solex), 'r.-')
legend('Paso h = 1', 'Paso h = 0.5')
hold off



%%
%
% FIN del Problema 5.2
