%% 6.2. Condicionamiento de un sistema lineal.
% 
% En este problema se interpreta grficmente el mal condicionamiento de una
% matriz y se justifica la necesidad de utilizar el pivotamiento parcial
% para evitar que un sistema bien condicionado se haga mal condicionado.
%
%
% Tema 6, Problema 2
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Consideramos el sistema de ecuaciones lineales
%
%           3x +       y = 7
%           3x + 1.0001y = 7

A = [3 1; 3 1.0001]

b = [7 7]'


%%
%
% Cada ecuacin se representa mediante una recta en el plano.

x = linspace(1,3,50); 
y1 = 7 - 3*x;
y2 = (7 - 3*x)/1.01; % (Cambio el coeficiente para que se distingan las rectas en la grfica)
plot(x, y1, x, y2, 'r')
title('Sistema mal condicionado (compatible determinado)')
legend('3x +       y = 7', '3x + 1.0001y = 7')
grid

%%
%
% La solucin del sistema es el punto de interseccin de las rectas.

sol = A\b

hold on
plot(sol(1), sol(2), 'g*')
hold off


%%
%
% El punto de interseccin puede variar mucho al mover un poco las rectas.
% Se trata de un sistema mal condicionado pues la solucin es muy sensible
% al error de los datos y operaciones.
% El nmero de condicin de la matriz del sistema es excesivamente grande
% para una matriz de tamao 22. 

cond(A)


%%
%
% En efecto, si cambiamos el trmino independiente de la segunda ecuacin a
% 6.9999, obtenemos un sistema
%           3x +       y = 7
%           3x + 1.0001y = 6.9999
% cuya solucin es completamente distinta a la original.

b(2) = b(2) - 0.0001
sol2 = A\b

y2 = (6.99 - 3*x)/1.01;
plot(x, y1, x, y2, 'r', sol(1), sol(2), 'g*', sol2(1), sol2(2), 'y*')
title('Sistema mal condicionado. Variacin de la solucin')
legend('3x +       y = 7', '3x + 1.0001y = 6.9999')
grid



%%
%
% Otras pequeas variaciones modifican incluso el carcter del sistema. Por
% ejemplo, haciendo el coeficiente de y en la segunda ecuacin igual a 1,
% se obtiene un sistema incompatible. 
%
%           3x + y = 7
%           3x + y = 6.9999

y2 = 6.95 - 3*x;
plot(x, y1, x, y2, 'r', sol(1), sol(2), 'g*', sol2(1), sol2(2), 'y*')
title('Sistema mal condicionado (incompatible)')
legend('3x + y = 7', '3x + y = 6.9999')
axis([1 3 -2 4])
grid


%%
%
% Cambiando a 7 el trmino independiente de la segunda ecuacin de este
% ltimo sistema, se hace indeterminado.
%
%           3x + y = 7
%           3x + y = 7

y2 = 7 - 3*x;
plot(x, y1, x, y2, 'r+', sol(1), sol(2), 'g*', sol2(1), sol2(2), 'y*')
title('Sistema mal condicionado (indeterminado)')
legend('3x + y = 7', '3x + y = 7')
grid


%%
%
% El sistema de ecuaciones 
%
%           0.003x + y = 1.006
%               3x + y = 7 
%
% est bien condicionado. Su nmero de condicin es notablemente menor que
% el del anterior sistema. 

A = [0.003 1; 3 1]
b = [1.006 7]'
cond(A)

y2 = 1.06 - 0.03*x; % (Cambio el coeficiente para que se distingan las rectas en la grfica)
plot(x, y2, x, y1, 'r')
title('Sistema bien condicionado')
legend('0.003x + y = 1.006', '3x + y = 7')
grid


%%
%
% La solucin del sistema es el punto de interseccin de las rectas.

sol = A\b

hold on
plot(sol(1), sol(2), 'g*')
hold off


%%
%
% Ahora bien, aunque el mal condicionamiento de un sistema es imposible de
% cambiar, no ocurre lo mismo con el buen condicionamiento. Un algoritmo de
% eliminacin deficiente puede arruinar un sistema bien condicionado. Por
% ejemplo, si eliminamos la x de la segunda ecuacin, obtenemos un sistema
% mal condicionado.

A(2,:) = A(2,:) - A(2,1)/A(1,1)*A(1,:)
b(2) = b(2) - A(2,1)/A(1,1)*b(1)

cond(A)


%%
%
% Las rectas que representan las ecuaciones son casi paralelas. Este
% empeoramiento es debido a que el pivote utilizado es mil veces menor que
% el coeficiente a eliminar.

uno = ones(size(x));
v = axis;
plot(x, y2, x, uno, 'r', sol(1), sol(2), 'g*')
title('Sistema degradado por eliminar sin pivotacin')
legend('0.003x + y = 1.006', 'y = 1')
axis(v)
grid


%%
%
% Para evitar esto, debemos eliminar con pivotacin parcial. 
% Recuperamos el sistema inicial.

A = [0.003 1; 3 1]
b = [1.006 7]'

plot(x, y2, x, y1, 'r')
title('Sistema bien condicionado')
legend('0.003x + y = 1.006', '3x + y = 7')
grid


%%
%
% Intercambiamos las ecuaciones para utilizar como pivote el elemento de
% mayor valor absoluto de la columna.

A = A([2 1], :)
b = b([2 1], :)

plot(x, y1, x, y2, 'r')
title('Sistema permutado con pivotacin parcial')
legend('3x + y = 7', '0.003x + y = 1.006')
grid


%%
%
% Al eliminar ahora se obtiene un sistema que sigue siendo bien
% condicionado.

A(2,:) = A(2,:) - A(2,1)/A(1,1)*A(1,:)
b(2) = b(2) - A(2,1)/A(1,1)*b(1)
cond(A)

plot(x, y1, x, uno, 'r', sol(1), sol(2), 'g*')
title('Sistema triangularizado con pivotacin parcial')
legend('x + y = 7', 'y = 1')
grid


%%
%
% El sistema de ecuaciones
%
%           2.01x +   y + 2z = 5.01
%              2x +  3y + 2z = 7
%              4x + 16y + 4z = 24
% 
% es mal condicionado.

A = [2.01  1  2
     2     3  2
     4    16  4];
 
b = [5.01 7 24]';

cond(A)

x = 0:0.1:2; y = x;
[X,Y] = meshgrid(x,y);
Z1 = (5.01 - 2.01*X - Y)/2;
Z2 = (7 - 2*X -3*Y)/2;
Z3 = (24 -4*X -16*Y)/4;
surf(X, Y, Z1), hold on
surf(X, Y, Z2)
surf(X, Y, Z3)
view(140, 30)
title('Interseccin de tres planos')
xlabel('x')
ylabel('y')


%%
%
% La solucin del sistema es la interseccin de los tres planos que
% definen sus ecuaciones. Como son casi paralelos, la interseccin es muy
% sensible a pequeos cambios.

sol = A\b

plot3(sol(1), sol(2), sol(3), 'r*')
hold off


%%
%
% Cambiando el coeficiente de x en la primera ecuacin de 2.01 a 2.001, la
% solucin cambia totalmente.

A

A(1,1) = 2.001

sol2 =A\b

x = -11:2:11; y = x;
[X,Y] = meshgrid(x,y);
Z1 = (5.01 - 2.01*X - Y)/2;
Z2 = (7 - 2*X -3*Y)/2;
Z3 = (24 -4*X -16*Y)/4;
surf(X, Y, Z1), hold on
surf(X, Y, Z2)
surf(X, Y, Z3)
view(140, 30)
axis([-11 11 -11 11 -60 60])
title('Variacin de la solucin')
xlabel('x')
ylabel('y')
plot3(sol(1), sol(2), sol(3), 'r*')
plot3(sol2(1), sol2(2), sol2(3), 'y*')
hold off


%%
%
% Al hacer dicho coeficiente igual a 1, el sistema resulta incompatible.
% Los planos se cortan en rectas paralelas.

A(1,1) = 1;
rank(A)
rank([A b])

x = 0.99:0.002:1.005; y = x;
[X,Y] = meshgrid(x,y);
Z1 = (5.01 - 2*X - Y)/2;
Z2 = (7 - 2*X -3*Y)/2;
Z3 = (24 -4*X -16*Y)/4;
surf(X, Y, Z1), hold on
surf(X, Y, Z2)
surf(X, Y, Z3)
plot3(sol(1), sol(2), sol(3), 'r*')
hold off
view(240, 30)
title('Planos que se cortan 2 a 2 en rectas paralelas')
xlabel('x')
ylabel('y')


%%
%
% FIN del Problema 6.2