%% 6.5. Convergencia de los mtodos iterativos.
% 
% Este problema muestra un sistema de ecuaciones lineales que no puede
% resolverse iterativamente por el mtodo de Jacobi, mientras que los
% mtodos de Gauss-Seidel y sobrerrelajacin convergen.
%
%
% Tema 6, Problema 5
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% Construimos la matriz del sistema para el caso de n = 9.

n = 9;
for i = 1:n
    for j = 1:n
        if i <= j
            A(i,j) = i*(n-j+1);
        else
            A(i,j) = A(j,i);
        end
    end
end
A

imagesc(A)
axis equal off
title('Representacin de la matriz del sistema')


%%
% La columna de trminos independientes coincide con la ltima columna de
% la matriz del sistema.

b = (1:9)'

imagesc([A zeros(9,1) b])
axis equal off
title('Representacin de la matriz ampliada')


%%
%
% El sistema tiene una solucin obvia x = (0, 0, ..., 0, 1) que  obtenemos
% en MATLAB mediante la contrabarra \:

sol = A\b


%%
%
% Para resolver el sistema por el mtodo de Jacobi, construimos las
% matrices que intervienen en la iteracin:

M = diag(diag(A))
N = M - A

P = inv(M)*N % No se usa explcitamente en los clculos. Slo para tus ojos.
imagesc(-P)
axis equal off
title('Representacin de la matriz de iteracin de Jacobi')


%%
%
% Efectuamos 5 iteraciones, segn M*x(k) = N*x(k-1) + b a partir de una
% estimacin inicial cercana a la solucion y observamos que el mtodo de
% Jacobi diverge.

x0 = sol + 0.1*randn(n,1);
x = x0;
X = x;
for k = 1:5
    x = M\(N*x+b);
    X = [X,x];
end
X

plot(X')
title('Evolucin de los iterados de Jacobi')
xlabel('Iteracin')
ylabel('Valor de las componentes')
legend('x_1', 'x_2', 'x_3', 'x_4', 'x_5', 'x_6', 'x_7', 'x_8', 'x_9', ...
    'Location', 'BestOutside')
grid


%%
%
% Las matrices que intervienen en las iteraciones del mtodo de
% Gauss-Seidel se obtienen con

M = tril(A)
N = M - A

P = inv(M)*N
imagesc(P)
axis equal off
title('Representacin de la matriz de iteracin de Gauss-Seidel')


%%
%
%
% Establecemos como criterio de convergencia que dos iterados difieran
% menos de 0,001 en la norma infinito (que es el mximo valor absoluto de
% las componentes). A diferencia del mtodo de Jacobi, la iteracin de
% Gauss-Seidel converge.

x0 = randn(n,1);
x = x0;
incr = 1;
k = 1
while incr > 0.001 & k < 200
    x(:,k+1) = M\(N*x(:,k) + b);
    incr = norm(x(:,k+1) - x(:,k), inf)
    k = k+1
end
iterseidel = k-1
x

plot(x')
title('Evolucin de los iterados de Gauss-Seidel')
xlabel('Iteracin')
ylabel('Valor de las componentes')
legend('x_1', 'x_2', 'x_3', 'x_4', 'x_5', 'x_6', 'x_7', 'x_8', 'x_9', ...
    'Location', 'BestOutside')
grid

%%
%
% La distancia en norma infinito entre la aproximacin de Gauss-Seidel y la
% solucin exacta es mayor pero del orden de 0.001.

norm(x(:,k) - sol)


%%
%
%
% Las matrices de iteracin del mtodo de sobrerrelajacin se generan con MATLAB para
% un valor dado de w.

w = 1.5;
D = diag(diag(A));
L = tril(A,-1);
M = L + D/w
N = M - A

P = inv(M)*N
imagesc(P)
axis equal off
title('Representacin de la matriz de iteracin de sobrerrelajacin')


%%
%
% Iteramos con estas matrices, esperando una convergencia ms rpida que
% por el mtodo de Gauss-Seidel.

x = x0;
incr = 1;
k = 1;
while incr > 0.001 & k < 200
    x(:,k+1) = M\(N*x(:,k) + b);
    incr = norm(x(:,k+1) - x(:,k), inf)
    k = k+1
end
x

plot(x')
title('Evolucin de los iterados de sobrerrelajacin con factor w = 1.5')
xlabel('Iteracin')
ylabel('Valor de las componentes')
legend('x_1', 'x_2', 'x_3', 'x_4', 'x_5', 'x_6', 'x_7', 'x_8', 'x_9', ...
    'Location', 'BestOutside')
grid


%%
%
% La distancia en norma infinito entre la aproximacin obtenida por el
% mtodo de sobrerrelajacin y la solucin exacta es mayor pero del orden
% de 0.001.

norm(x(:,k) - sol)


%%
%
% El mtodo de sobrerrelajacin no mejora en este ejemplo los resultados de
% Gauss-Seidel, necesitando ms iteraciones para obtener la precisin
% deseada, como comprobamos para distintos valores de w.
 
for k = 1:9
    w(k) = 0.2*k;
    [x, incr, iter(k)] = sobrerrel(A, b, x0, w(k), 0.001, 200);
end
iter

stem(w, iter, 'filled')
title('Mtodo de sobrerrelajacin')
xlabel('Factor de relajacin')
ylabel('Iteraciones')
grid
hold on
stem(1, iterseidel, 'r', 'filled')
legend('Sobrerrelajacin', 'Gauss-Seidel', 'Location', 'North')

    
%%
%
% FIN del Problema 6.5