%% 6.9. Tubo de refrigeracin.
% 
% En este problema se discretiza la ecuacin del calor un tubo de
% refrigeracin de seccin cuadrada, semisumergido en un lquido a menor
% temperatura y se resuelve el sistema lineal resultante por mtodos
% directos e iterativos.
%
%
% Tema 6, Problema 9
% Problemas resueltos de Mtodos Numricos
% A. Cordero, J.L. Hueso, E. Martnez, J.R. Torregrosa
% Ed. Thomson


%% 
%
% La figura muestra la malla de celdas cuadradas de lado 1cm superpuesta a
% la seccin del tubo. Esta malla determina 11  11 nodos que estn
% marcados en blanco, si la temperatura en ellos es conocida y en negro si
% es desconocida. Suponemos que la temperatura en cada nodo, Tij es la
% media de las temperaturas de los 4 nodos vecinos.

image(imread('tubo.jpg'))
axis equal off
title('Nodos de discretizacin en la seccin del tubo')


%%
%
% Al aplicar esta condicin a cada uno de los nodos obtenemos un sistema de ecuaciones lineales.
% Las temperaturas conocidas pasan al segundo miembro formando los trminos independientes del
% sistema. Ignorando inicialmene el hueco, generamos la matriz del sistema a partir de la matriz de la
% ecuacin del calor en una placa 99.

n = 9; m = 9;
A = -calor2D(n,m);

spy(A)
title('Estructura de la matriz A para una malla 99')
set(gca, 'XTick', 9:9:81, 'YTick', 9:9:81, ...
    'XMinorTick', 'on', 'YMinorTick', 'on')
hold on
for k = 0 : m : m*n
    plot([0.5, m*n + 0.5], [k + 0.5, k + 0.5], 'r')
    plot([k + 0.5, k + 0.5], [0.5, m*n + 0.5], 'r')
end
hold off

%%
%
% De esta matriz, hay que eliminar las filas y columnas correspondientes a
% los 5  5 nodos del hueco del tubo. Los nodos de la placa se numeran por
% columnas para generar el sistema de ecuaciones lineales. Para ver qu
% ecuaciones corresponden a los nodos del hueco, lo simulamos mediante
% ceros en una matriz T de unos y buscamos los ceros en dicha matriz,
% previamente encolumnada. Las posiciones encontradas corresponden a las
% ecuaciones y variables a eliminar.

% Hueco
Tubo = ones(9,9);
Tubo(3:7,3:7) = zeros(5,5);
tubocol = Tubo(:);
elim = find(tubocol==0);
A(elim,:) = [ ];
A(:,elim) = [ ];

spy(A)
title('Estructura de la matriz A para una malla 99 con un hueco 55')
set(gca, 'XMinorTick', 'on', 'YMinorTick', 'on')
    

%%
%
% Generamos ahora una matriz que contiene las temperaturas dadas dispuestas
% como en la malla. Los nodos incgnita estn a 0.

% Condiciones de contorno
C = zeros(11,11);
C(1,:) = 100;       % Cara superior
C(11,:) = 0;        % Base
lin = linspace(100,0,6);
C(1:6,1) = lin';    % Lateral izquierdo
C(1:6,11) = lin';   % Lateral derecho
C(4:8,4:8) = 200    % Interior del tubo

imagesc(C)
colorbar
axis equal off
title('Representacin de las temperaturas conocidas')


%%
%
% Los trminos independientes se obtienen sumando para cada nodo las
% temperaturas conocidas de los nodos vecinos.

% Trminos independientes
uno = 1:9;
dos = 2:10;
tres = 3:11;
B = C(uno, dos) + C(dos, uno) + C(dos, tres) + C(tres, dos);


%%
%
% Disponemos los trminos independientes en columna, eliminamos las
% componentes correspondientes a los nodos del hueco y resolvemos el
% sistema.

b = B(:);
b(elim) = [ ];
n = length(b);
Tcol = A\b;

%%
%
% Finalmente, dispondremos los resultados obtenidos sobre la matriz C que
% contiene las condiciones de contorno, para obtener una representacin
% grfica de la solucin. Obtenemos con find las posiciones de los nodos
% incgnita y creamos con sparse una matriz T con los valores de la
% solucin en dichas posiciones.

% Posicin en la placa de los nodos variables
[I,J] = find(Tubo);
T = sparse(I, J, Tcol);
%C(dos, dos) = C(dos, dos) + T
C(dos, dos) = T;
C(4:8,4:8) = 200;    % Interior del tubo

imagesc(C)
colorbar
axis equal off
title('Representacin de las temperaturas en la seccin del tubo')


%%
%
% En la representacin anterior prevalece el caracter discreto de la
% solucin obtenida. En cambio, las curvas de nivel nos dan una mejor idea
% de la solucin continua en el dominio.

[c,h] = contour(C);
clabel(c,h,'labelspacing',400);
axis ij
axis equal
axis ([1 11 1 11])
title('Isotermas de la seccin del tubo')


%%
%
% Resolvemos ahora el sistema por mtodos iterativos, tomando como
% estimacin inicial una temperatura de 100C en todos los nodos incgnita
% e iteramos hasta que la diferencia entre dos estimaciones de la
% temperatura en cada nodo sea menor que 10^-3.

u0 = 100*ones(n,1);
tol = 1e-3;

T = sparse(I, J, u0);
C(dos, dos) = T;
C(4:8,4:8) = 200;
imagesc(C)
colorbar
axis equal off
title('Mtodos iterativos. Estimacin inicial')

    
%%
%
% Iteramos en primer lugar por el mtodo de Jacobi.

iter = 0;
incr = tol +1;
u = u0;
while incr > tol & iter < 100
    [u, incr, k] = jacobi(A, b, u, tol, 1);
    iter = iter + 1;
    T = sparse(I, J, u);
    C(dos, dos) = T;
    C(4:8,4:8) = 200;
    imagesc(C)
    colorbar
    axis equal off
    title(['Mtodo de Jacobi. Paso ', num2str(iter)])
    pause(1/1.2^iter) 
end


%%
%
% La convergencia del mtodo de Jacobi es muy lenta, por lo que la
% diferencia entre la iteracin final y la solucin obtenida mediante la
% contrabarra es mayor que la tolerancia requerida.

disp(' ')
disp(['Nmero de iteraciones: ' num2str(iter)])
disp(' ')
disp(['Diferencia con la solucin exacta: ', num2str(norm(u-Tcol, inf))])


%%
%
% En segundo lugar, iteramos por el mtodo de Gauss-Seidel.

iter = 0;
incr = tol +1;
u = u0;
while incr > tol & iter < 100
    [u, incr, k] = gseidel(A, b, u, tol, 1);
    iter = iter + 1;
    T = sparse(I, J, u);
    C(dos, dos) = T;
    C(4:8,4:8) = 200;
    imagesc(C)
    colorbar
    axis equal off
    title(['Mtodo de Gauss-Seidel. Paso ', num2str(iter)])
    pause(1/1.2^iter) 
end


%%
%
% Gauss-Seidel converge aproximadamente con la mitad de iteraciones que
% Jacobi. La diferencia entre la iteracin final y la solucin es muy
% aproximada a la tolerancia requerida.

disp(' ')
disp(['Nmero de iteraciones: ' num2str(iter)])
disp(' ')
disp(['Diferencia con la solucin exacta: ', num2str(norm(u-Tcol, inf))])


%%
%
% Resolvemos ahora el sistema por sobrerrelajacin, con factor w = 1.3.

w = 1.3;
iter = 0;
incr = tol +1;
u = u0;
while incr > tol & iter < 100
    [u, incr, k] = sobrerrel(A, b, u, w, tol, 1);
    iter = iter + 1;
    T = sparse(I, J, u);
    C(dos, dos) = T;
    C(4:8,4:8) = 200;
    imagesc(C)
    colorbar
    axis equal off
    title(['Mtodo de sobrerrelajacin. Paso ', num2str(iter)])
    pause(1/1.2^iter) 
end


%%
%
% Para w = 1.3, la iteracin de sobrerrelajacin converge ms rpidamente
% que las anteriores. El nmero de incgnitas en este problema es de 56,
% por lo que los mtodos iterativos son competitivos frente a la solucin
% directa porque requieren menos de 56 iteraciones (muchas menos en el caso
% de sobrerrelajacin con factor w ptimo).

disp(' ')
disp(['Nmero de iteraciones: ' num2str(iter)])
disp(' ')
disp(['Diferencia con la solucin exacta: ', num2str(norm(u-Tcol, inf))])


%%
%
% FIN del Problema 6.9